@@ -1,3 +1,6 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "iputils.hh"
 #include <sys/socket.h> 
 
@@ -20,16 +23,20 @@ int SSocket(int family, int type, int flags)
 int SConnect(int sockfd, const ComboAddress& remote)
 {
   int ret = connect(sockfd, (struct sockaddr*)&remote, remote.getSocklen());
-  if(ret < 0)
-    RuntimeError(boost::format("connecting socket to %s: %s") % remote.toStringWithPort() % strerror(errno));
+  if(ret < 0) {
+    int savederrno = errno;
+    RuntimeError(boost::format("connecting socket to %s: %s") % remote.toStringWithPort() % strerror(savederrno));
+  }
   return ret;
 }
 
 int SBind(int sockfd, const ComboAddress& local)
 {
   int ret = bind(sockfd, (struct sockaddr*)&local, local.getSocklen());
-  if(ret < 0)
-    RuntimeError(boost::format("binding socket to %s: %s") % local.toStringWithPort() % strerror(errno));
+  if(ret < 0) {
+    int savederrno = errno;
+    RuntimeError(boost::format("binding socket to %s: %s") % local.toStringWithPort() % strerror(savederrno));
+  }
   return ret;
 }
 
@@ -115,8 +122,7 @@ bool IsAnyAddress(const ComboAddress& addr)
   return false;
 }
 
-// FIXME: this function is unused, and using it could reduce some code duplication
-int sendfromto(int sock, const char* data, int len, int flags, const ComboAddress& from, const ComboAddress& to)
+ssize_t sendfromto(int sock, const char* data, size_t len, int flags, const ComboAddress& from, const ComboAddress& to)
 {
   struct msghdr msgh;
   struct iovec iov;
@@ -132,7 +138,7 @@ int sendfromto(int sock, const char* data, int len, int flags, const ComboAddres
   msgh.msg_namelen = to.getSocklen();
 
   if(from.sin4.sin_family) {
-    addCMsgSrcAddr(&msgh, cbuf, &from);
+    addCMsgSrcAddr(&msgh, cbuf, &from, 0);
   }
   else {
     msgh.msg_control=NULL;
@@ -158,3 +164,76 @@ void fillMSGHdr(struct msghdr* msgh, struct iovec* iov, char* cbuf, size_t cbufs
   msgh->msg_iovlen = 1;
   msgh->msg_flags = 0;
 }
+
+void ComboAddress::truncate(unsigned int bits)
+{
+  uint8_t* start;
+  int len=4;
+  if(sin4.sin_family==AF_INET) {
+    if(bits >= 32)
+      return;
+    start = (uint8_t*)&sin4.sin_addr.s_addr;
+    len=4;
+  }
+  else {
+    if(bits >= 128)
+      return;
+    start = (uint8_t*)&sin6.sin6_addr.s6_addr;
+    len=16;
+  }
+
+  auto tozero= len*8 - bits; // if set to 22, this will clear 1 byte, as it should
+
+  memset(start + len - tozero/8, 0, tozero/8); // blot out the whole bytes on the right
+  
+  auto bitsleft=tozero % 8; // 2 bits left to clear
+
+  // a b c d, to truncate to 22 bits, we just zeroed 'd' and need to zero 2 bits from c
+  // so and by '11111100', which is ~((1<<2)-1)  = ~3
+  uint8_t* place = start + len - 1 - tozero/8; 
+  *place &= (~((1<<bitsleft)-1));
+}
+
+ssize_t sendMsgWithTimeout(int fd, const char* buffer, size_t len, int timeout, ComboAddress& dest, const ComboAddress& local, unsigned int localItf)
+{
+  struct msghdr msgh;
+  struct iovec iov;
+  char cbuf[256];
+  bool firstTry = true;
+  fillMSGHdr(&msgh, &iov, cbuf, sizeof(cbuf), const_cast<char*>(buffer), len, &dest);
+  addCMsgSrcAddr(&msgh, cbuf, &local, localItf);
+
+  do {
+    ssize_t written = sendmsg(fd, &msgh, 0);
+
+    if (written > 0)
+      return written;
+
+    if (errno == EAGAIN) {
+      if (firstTry) {
+        int res = waitForRWData(fd, false, timeout, 0);
+        if (res > 0) {
+          /* there is room available */
+          firstTry = false;
+        }
+        else if (res == 0) {
+          throw runtime_error("Timeout while waiting to write data");
+        } else {
+          throw runtime_error("Error while waiting for room to write data");
+        }
+      }
+      else {
+        throw runtime_error("Timeout while waiting to write data");
+      }
+    }
+    else {
+      unixDie("failed in write2WithTimeout");
+    }
+  }
+  while (firstTry);
+
+  return 0;
+}
+
+template class NetmaskTree<bool>;
+
