@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2002 - 2014 PowerDNS.COM BV
+    Copyright (C) 2002 - 2015 PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 as 
@@ -21,6 +21,9 @@
 */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "utility.hh"
 #include "lwres.hh"
 #include <iostream>
@@ -44,15 +47,17 @@
 #include "dns_random.hh"
 #include <boost/scoped_array.hpp>
 #include <boost/algorithm/string.hpp>
+#include "validate-recursor.hh"
+#include "ednssubnet.hh"
 
 //! returns -2 for OS limits error, -1 for permanent error that has to do with remote **transport**, 0 for timeout, 1 for success
 /** lwr is only filled out in case 1 was returned, and even when returning 1 for 'success', lwr might contain DNS errors
     Never throws! 
  */
-int asyncresolve(const ComboAddress& ip, const string& domain, int type, bool doTCP, bool sendRDQuery, int EDNS0Level, struct timeval* now, LWResult *lwr)
+int asyncresolve(const ComboAddress& ip, const DNSName& domain, int type, bool doTCP, bool sendRDQuery, int EDNS0Level, struct timeval* now, boost::optional<Netmask>& srcmask, LWResult *lwr)
 {
-  int len; 
-  int bufsize=1500;
+  size_t len;
+  size_t bufsize=g_outgoingEDNSBufsize;
   scoped_array<unsigned char> buf(new unsigned char[bufsize]);
   vector<uint8_t> vpacket;
   //  string mapped0x20=dns0x20(domain);
@@ -60,23 +65,35 @@ int asyncresolve(const ComboAddress& ip, const string& domain, int type, bool do
 
   pw.getHeader()->rd=sendRDQuery;
   pw.getHeader()->id=dns_random(0xffff);
-  
+  /* RFC 6840 section 5.9:
+   *  This document further specifies that validating resolvers SHOULD set
+   *  the CD bit on every upstream query.  This is regardless of whether
+   *  the CD bit was set on the incoming query [...]
+   *
+   * sendRDQuery is only true if the qname is part of a forward-zone-recurse (or
+   * set in the forward-zone-file), so we use this as an indicator for it being
+   * an "upstream query". To stay true to "dnssec=off means 3.X behaviour", we
+   * only set +CD on forwarded query in any mode other than dnssec=off.
+   */
+  pw.getHeader()->cd=(sendRDQuery && g_dnssecmode != DNSSECMode::Off);
+
   string ping;
-
-  if(EDNS0Level && !doTCP) {
+  bool weWantEDNSSubnet=false;
+  if(EDNS0Level) { 
     DNSPacketWriter::optvect_t opts;
-    if(EDNS0Level > 1) {
-      uint32_t nonce=dns_random(0xffffffff);
-      ping.assign((char*) &nonce, 4);
-
-      opts.push_back(make_pair(5, ping));
+    if(srcmask) {
+      EDNSSubnetOpts eo;
+      eo.source = *srcmask;
+      //      cout<<"Adding request mask: "<<eo.source.toString()<<endl;
+      opts.push_back(make_pair(8, makeEDNSSubnetOptsString(eo)));
+      srcmask=boost::optional<Netmask>(); // this is also our return value
+      weWantEDNSSubnet=true;
     }
 
-    pw.addOpt(1200, 0, 0, opts); // 1200 bytes answer size
+    pw.addOpt(g_outgoingEDNSBufsize, 0, g_dnssecmode == DNSSECMode::Off ? 0 : EDNSOpts::DNSSECOK, opts); 
     pw.commit();
   }
   lwr->d_rcode = 0;
-  lwr->d_pingCorrect = false;
   lwr->d_haveEDNS = false;
   int ret;
 
@@ -89,7 +106,7 @@ int asyncresolve(const ComboAddress& ip, const string& domain, int type, bool do
     if(ip.sin4.sin_family==AF_INET6)
       g_stats.ipv6queries++;
 
-    if((ret=asendto((const char*)&*vpacket.begin(), (int)vpacket.size(), 0, ip, pw.getHeader()->id, 
+    if((ret=asendto((const char*)&*vpacket.begin(), vpacket.size(), 0, ip, pw.getHeader()->id,
                     domain, type, &queryfd)) < 0) {
       return ret; // passes back the -2 EMFILE
     }
@@ -126,7 +143,7 @@ int asyncresolve(const ComboAddress& ip, const string& domain, int type, bool do
       if(!(ret > 0))
         return ret;
       
-      memcpy(&tlen, packet.c_str(), 2);
+      memcpy(&tlen, packet.c_str(), sizeof(tlen));
       len=ntohs(tlen); // switch to the 'len' shared with the rest of the function
       
       ret=arecvtcp(packet, len, &s, false);
@@ -154,7 +171,7 @@ int asyncresolve(const ComboAddress& ip, const string& domain, int type, bool do
   if(ret <= 0) // includes 'timeout'
     return ret;
 
-  lwr->d_result.clear();
+  lwr->d_records.clear();
   try {
     lwr->d_tcbit=0;
     MOADNSParser mdp((const char*)buf.get(), len);
@@ -166,33 +183,30 @@ int asyncresolve(const ComboAddress& ip, const string& domain, int type, bool do
       return 1; // this is "success", the error is set in lwr->d_rcode
     }
 
-    if(!pdns_iequals(domain, mdp.d_qname)) { 
-      if(!mdp.d_qname.empty() && domain.find((char)0) == string::npos) {// embedded nulls are too noisy, plus empty domains are too
+    if(domain != mdp.d_qname) { 
+      if(!mdp.d_qname.empty() && domain.toString().find((char)0) == string::npos /* ugly */) {// embedded nulls are too noisy, plus empty domains are too
         L<<Logger::Notice<<"Packet purporting to come from remote server "<<ip.toString()<<" contained wrong answer: '" << domain << "' != '" << mdp.d_qname << "'" << endl;
       }
       // unexpected count has already been done @ pdns_recursor.cc
       goto out;
     }
-
-    for(MOADNSParser::answers_t::const_iterator i=mdp.d_answers.begin(); i!=mdp.d_answers.end(); ++i) {          
-      DNSResourceRecord rr;
-      rr.qtype=i->first.d_type;
-      rr.qname=i->first.d_label;
-      rr.ttl=i->first.d_ttl;
-      rr.content=i->first.d_content->getZoneRepresentation();  // this should be the serialised form
-      rr.d_place=(DNSResourceRecord::Place) i->first.d_place;
-      lwr->d_result.push_back(rr);
-    }
+    
+    for(const auto& a : mdp.d_answers)
+      lwr->d_records.push_back(a.first);
 
     EDNSOpts edo;
-    if(EDNS0Level > 1 && getEDNSOpts(mdp, &edo)) {
+    if(EDNS0Level > 0 && getEDNSOpts(mdp, &edo)) {
       lwr->d_haveEDNS = true;
-      for(vector<pair<uint16_t, string> >::const_iterator iter = edo.d_options.begin();
-          iter != edo.d_options.end(); 
-          ++iter) {
-        if(iter->first == 5 || iter->first == 4) {// 'EDNS PING'
-          if(iter->second == ping)  {
-            lwr->d_pingCorrect = true;
+
+      if(weWantEDNSSubnet) {
+        for(const auto& opt : edo.d_options) {
+          if(opt.first==8) {
+            EDNSSubnetOpts reso;
+            if(getEDNSSubnetOptsFromString(opt.second, &reso)) {
+              //	    cerr<<"EDNS Subnet response: "<<reso.source.toString()<<", scope: "<<reso.scope.toString()<<", family = "<<reso.scope.getNetwork().sin4.sin_family<<endl;
+              if(reso.scope.getBits())
+                srcmask = reso.scope;
+            }
           }
         }
       }
@@ -220,4 +234,3 @@ int asyncresolve(const ComboAddress& ip, const string& domain, int type, bool do
   return -1;
 }
 
-
