@@ -72,7 +72,6 @@ const char* g_pubsuffix[]={
 "logistics.aero",
 "magazine.aero",
 "maintenance.aero",
-"marketplace.aero",
 "media.aero",
 "microlight.aero",
 "modelling.aero",
@@ -95,7 +94,6 @@ const char* g_pubsuffix[]={
 "skydiving.aero",
 "software.aero",
 "student.aero",
-"taxi.aero",
 "trader.aero",
 "trading.aero",
 "trainer.aero",
@@ -122,10 +120,6 @@ const char* g_pubsuffix[]={
 "mil.al",
 "net.al",
 "org.al",
-"com.an",
-"net.an",
-"org.an",
-"edu.an",
 "ed.ao",
 "gv.ao",
 "og.ao",
@@ -196,16 +190,12 @@ const char* g_pubsuffix[]={
 "name.az",
 "pro.az",
 "biz.az",
-"org.ba",
-"net.ba",
+"com.ba",
 "edu.ba",
 "gov.ba",
 "mil.ba",
-"unsa.ba",
-"unbi.ba",
-"co.ba",
-"com.ba",
-"rs.ba",
+"net.ba",
+"org.ba",
 "biz.bb",
 "co.bb",
 "com.bb",
@@ -481,6 +471,19 @@ const char* g_pubsuffix[]={
 "net.cw",
 "org.cw",
 "gov.cx",
+"ac.cy",
+"biz.cy",
+"com.cy",
+"ekloges.cy",
+"gov.cy",
+"ltd.cy",
+"name.cy",
+"net.cy",
+"org.cy",
+"parliament.cy",
+"press.cy",
+"pro.cy",
+"tm.cy",
 "com.dm",
 "net.dm",
 "org.dm",
@@ -624,7 +627,10 @@ const char* g_pubsuffix[]={
 "org.gt",
 "co.gy",
 "com.gy",
+"edu.gy",
+"gov.gy",
 "net.gy",
+"org.gy",
 "com.hk",
 "edu.hk",
 "gov.hk",
@@ -701,6 +707,14 @@ const char* g_pubsuffix[]={
 "sch.id",
 "web.id",
 "gov.ie",
+"ac.il",
+"co.il",
+"gov.il",
+"idf.il",
+"k12.il",
+"muni.il",
+"net.il",
+"org.il",
 "ac.im",
 "co.im",
 "com.im",
@@ -1985,7 +1999,6 @@ const char* g_pubsuffix[]={
 "kamiamakusa.kumamoto.jp",
 "kashima.kumamoto.jp",
 "kikuchi.kumamoto.jp",
-"kosa.kumamoto.jp",
 "kumamoto.kumamoto.jp",
 "mashiki.kumamoto.jp",
 "mifune.kumamoto.jp",
@@ -2070,7 +2083,6 @@ const char* g_pubsuffix[]={
 "kakuda.miyagi.jp",
 "kami.miyagi.jp",
 "kawasaki.miyagi.jp",
-"kesennuma.miyagi.jp",
 "marumori.miyagi.jp",
 "matsushima.miyagi.jp",
 "minamisanriku.miyagi.jp",
@@ -3019,6 +3031,7 @@ const char* g_pubsuffix[]={
 "edu.mg",
 "mil.mg",
 "com.mg",
+"co.mg",
 "com.mk",
 "org.mk",
 "net.mk",
@@ -3668,13 +3681,28 @@ const char* g_pubsuffix[]={
 "store.nf",
 "com.ng",
 "edu.ng",
+"gov.ng",
+"i.ng",
+"mil.ng",
+"mobi.ng",
 "name.ng",
 "net.ng",
 "org.ng",
 "sch.ng",
-"gov.ng",
-"mil.ng",
-"mobi.ng",
+"com.ni",
+"gob.ni",
+"edu.ni",
+"org.ni",
+"nom.ni",
+"net.ni",
+"mil.ni",
+"co.ni",
+"biz.ni",
+"web.ni",
+"int.ni",
+"ac.ni",
+"in.ni",
+"info.ni",
 "bv.nl",
 "fhs.no",
 "vgs.no",
@@ -4548,13 +4576,17 @@ const char* g_pubsuffix[]={
 "est.pr",
 "prof.pr",
 "ac.pr",
+"aaa.pro",
 "aca.pro",
+"acct.pro",
+"avocat.pro",
 "bar.pro",
 "cpa.pro",
+"eng.pro",
 "jur.pro",
 "law.pro",
 "med.pro",
-"eng.pro",
+"recht.pro",
 "edu.ps",
 "gov.ps",
 "sec.ps",
@@ -4591,26 +4623,26 @@ const char* g_pubsuffix[]={
 "net.qa",
 "org.qa",
 "sch.qa",
-"com.re",
 "asso.re",
+"com.re",
 "nom.re",
-"com.ro",
-"org.ro",
-"tm.ro",
-"nt.ro",
-"nom.ro",
-"info.ro",
-"rec.ro",
 "arts.ro",
+"com.ro",
 "firm.ro",
+"info.ro",
+"nom.ro",
+"nt.ro",
+"org.ro",
+"rec.ro",
 "store.ro",
+"tm.ro",
 "www.ro",
-"co.rs",
-"org.rs",
-"edu.rs",
 "ac.rs",
+"co.rs",
+"edu.rs",
 "gov.rs",
 "in.rs",
+"org.rs",
 "ac.ru",
 "com.ru",
 "edu.ru",
@@ -4727,7 +4759,6 @@ const char* g_pubsuffix[]={
 "k-uralsk.ru",
 "kustanai.ru",
 "kuzbass.ru",
-"magnitka.ru",
 "mytis.ru",
 "nakhodka.ru",
 "nkz.ru",
@@ -5095,8 +5126,6 @@ const char* g_pubsuffix[]={
 "zhytomyr.ua",
 "zp.ua",
 "zt.ua",
-"co.ua",
-"pp.ua",
 "co.ug",
 "or.ug",
 "ac.ug",
@@ -5397,45 +5426,82 @@ const char* g_pubsuffix[]={
 "org.ws",
 "gov.ws",
 "edu.ws",
+"ac.za",
+"agric.za",
+"alt.za",
+"co.za",
+"edu.za",
+"gov.za",
+"grondar.za",
+"law.za",
+"mil.za",
+"net.za",
+"ngo.za",
+"nis.za",
+"nom.za",
+"org.za",
+"school.za",
+"tm.za",
+"web.za",
+"ac.zm",
+"biz.zm",
+"co.zm",
+"com.zm",
+"edu.zm",
+"gov.zm",
+"info.zm",
+"mil.zm",
+"net.zm",
+"org.zm",
+"sch.zm",
+"beep.pl",
 "cloudfront.net",
+"compute.amazonaws.com",
 "ap-northeast-1.compute.amazonaws.com",
+"ap-northeast-2.compute.amazonaws.com",
 "ap-southeast-1.compute.amazonaws.com",
 "ap-southeast-2.compute.amazonaws.com",
-"cn-north-1.compute.amazonaws.cn",
-"compute.amazonaws.cn",
-"compute.amazonaws.com",
-"compute-1.amazonaws.com",
-"eu-west-1.compute.amazonaws.com",
 "eu-central-1.compute.amazonaws.com",
+"eu-west-1.compute.amazonaws.com",
 "sa-east-1.compute.amazonaws.com",
-"us-east-1.amazonaws.com",
 "us-gov-west-1.compute.amazonaws.com",
 "us-west-1.compute.amazonaws.com",
 "us-west-2.compute.amazonaws.com",
+"compute-1.amazonaws.com",
 "z-1.compute-1.amazonaws.com",
 "z-2.compute-1.amazonaws.com",
+"us-east-1.amazonaws.com",
+"compute.amazonaws.com.cn",
+"cn-north-1.compute.amazonaws.com.cn",
 "elasticbeanstalk.com",
 "elb.amazonaws.com",
 "s3.amazonaws.com",
-"s3-us-west-2.amazonaws.com",
-"s3-us-west-1.amazonaws.com",
-"s3-eu-west-1.amazonaws.com",
+"s3-ap-northeast-1.amazonaws.com",
+"s3-ap-northeast-2.amazonaws.com",
 "s3-ap-southeast-1.amazonaws.com",
 "s3-ap-southeast-2.amazonaws.com",
-"s3-ap-northeast-1.amazonaws.com",
+"s3-eu-central-1.amazonaws.com",
+"s3-eu-west-1.amazonaws.com",
+"s3-external-1.amazonaws.com",
+"s3-external-2.amazonaws.com",
+"s3-fips-us-gov-west-1.amazonaws.com",
 "s3-sa-east-1.amazonaws.com",
 "s3-us-gov-west-1.amazonaws.com",
-"s3-fips-us-gov-west-1.amazonaws.com",
-"s3-website-us-east-1.amazonaws.com",
-"s3-website-us-west-2.amazonaws.com",
-"s3-website-us-west-1.amazonaws.com",
-"s3-website-eu-west-1.amazonaws.com",
-"s3-website-ap-southeast-1.amazonaws.com",
-"s3-website-ap-southeast-2.amazonaws.com",
-"s3-website-ap-northeast-1.amazonaws.com",
-"s3-website-sa-east-1.amazonaws.com",
-"s3-website-us-gov-west-1.amazonaws.com",
+"s3-us-west-1.amazonaws.com",
+"s3-us-west-2.amazonaws.com",
+"s3.ap-northeast-2.amazonaws.com",
+"s3.cn-north-1.amazonaws.com.cn",
+"s3.eu-central-1.amazonaws.com",
+"on-aptible.com",
+"pimienta.org",
+"poivron.org",
+"potager.org",
+"sweetpepper.org",
+"myfritz.net",
+"backplaneapp.io",
 "betainabox.com",
+"boxfuse.io",
+"mycd.eu",
 "ae.org",
 "ar.com",
 "br.com",
@@ -5470,13 +5536,39 @@ const char* g_pubsuffix[]={
 "us.org",
 "co.com",
 "c.la",
+"certmgr.org",
+"xenapponazure.com",
+"virtueeldomein.nl",
 "cloudcontrolled.com",
 "cloudcontrolapp.com",
 "co.ca",
+"co.cz",
+"c.cdn77.org",
+"cdn77-ssl.net",
+"r.cdn77.net",
+"rsc.cdn77.org",
+"ssl.origin.cdn77-secure.org",
 "co.nl",
 "co.no",
+"realm.cz",
 "cupcake.is",
+"cyon.link",
+"cyon.site",
+"daplie.me",
+"biz.dk",
+"co.dk",
+"firm.dk",
+"reg.dk",
+"store.dk",
+"dedyn.io",
+"dnshome.de",
 "dreamhosters.com",
+"mydrobo.com",
+"drud.io",
+"drud.us",
+"duckdns.org",
+"dy.fi",
+"tunk.org",
 "dyndns-at-home.com",
 "dyndns-at-work.com",
 "dyndns-blog.com",
@@ -5756,6 +5848,8 @@ const char* g_pubsuffix[]={
 "webhop.org",
 "worse-than.tv",
 "writesthisblog.com",
+"dynv6.net",
+"e4.cz",
 "eu.org",
 "al.eu.org",
 "asso.eu.org",
@@ -5812,34 +5906,66 @@ const char* g_pubsuffix[]={
 "tr.eu.org",
 "uk.eu.org",
 "us.eu.org",
+"eu-1.evennode.com",
+"eu-2.evennode.com",
+"us-1.evennode.com",
+"us-2.evennode.com",
+"apps.fbsbx.com",
 "a.ssl.fastly.net",
 "b.ssl.fastly.net",
 "global.ssl.fastly.net",
 "a.prod.fastly.net",
 "global.prod.fastly.net",
+"fhapp.xyz",
 "firebaseapp.com",
 "flynnhub.com",
+"freebox-os.com",
+"freeboxos.com",
+"fbx-os.fr",
+"fbxos.fr",
+"freebox-os.fr",
+"freeboxos.fr",
 "service.gov.uk",
 "github.io",
 "githubusercontent.com",
+"githubcloud.com",
+"gist.githubcloud.com",
+"gitlab.io",
 "ro.com",
+"goip.de",
 "appspot.com",
 "blogspot.ae",
+"blogspot.al",
+"blogspot.am",
+"blogspot.ba",
 "blogspot.be",
+"blogspot.bg",
 "blogspot.bj",
 "blogspot.ca",
 "blogspot.cf",
 "blogspot.ch",
+"blogspot.cl",
 "blogspot.co.at",
+"blogspot.co.id",
 "blogspot.co.il",
+"blogspot.co.ke",
 "blogspot.co.nz",
 "blogspot.co.uk",
+"blogspot.co.za",
 "blogspot.com",
 "blogspot.com.ar",
 "blogspot.com.au",
 "blogspot.com.br",
+"blogspot.com.by",
+"blogspot.com.co",
+"blogspot.com.cy",
+"blogspot.com.ee",
+"blogspot.com.eg",
 "blogspot.com.es",
+"blogspot.com.mt",
+"blogspot.com.ng",
 "blogspot.com.tr",
+"blogspot.com.uy",
 "blogspot.cv",
 "blogspot.cz",
 "blogspot.de",
@@ -5848,65 +5974,217 @@ const char* g_pubsuffix[]={
 "blogspot.fr",
 "blogspot.gr",
 "blogspot.hk",
+"blogspot.hr",
 "blogspot.hu",
 "blogspot.ie",
 "blogspot.in",
+"blogspot.is",
 "blogspot.it",
 "blogspot.jp",
 "blogspot.kr",
+"blogspot.li",
+"blogspot.lt",
+"blogspot.lu",
+"blogspot.md",
+"blogspot.mk",
 "blogspot.mr",
 "blogspot.mx",
+"blogspot.my",
 "blogspot.nl",
 "blogspot.no",
+"blogspot.pe",
 "blogspot.pt",
+"blogspot.qa",
 "blogspot.re",
 "blogspot.ro",
+"blogspot.rs",
 "blogspot.ru",
 "blogspot.se",
 "blogspot.sg",
+"blogspot.si",
 "blogspot.sk",
+"blogspot.sn",
 "blogspot.td",
 "blogspot.tw",
+"blogspot.ug",
+"blogspot.vn",
+"cloudfunctions.net",
 "codespot.com",
 "googleapis.com",
 "googlecode.com",
 "pagespeedmobilizer.com",
 "withgoogle.com",
+"withyoutube.com",
+"hashbang.sh",
+"hasura-app.io",
+"hepforge.org",
 "herokuapp.com",
 "herokussl.com",
 "iki.fi",
 "biz.at",
 "info.at",
+"meteorapp.com",
+"eu.meteorapp.com",
 "co.pl",
 "azurewebsites.net",
 "azure-mobile.net",
 "cloudapp.net",
+"bmoattachments.org",
 "4u.com",
+"ngrok.io",
 "nfshost.com",
+"nsupdate.info",
+"nerdpol.ovh",
+"blogsyte.com",
+"brasilia.me",
+"cable-modem.org",
+"ciscofreak.com",
+"collegefan.org",
+"couchpotatofries.org",
+"damnserver.com",
+"ddns.me",
+"ditchyourip.com",
+"dnsfor.me",
+"dnsiskinky.com",
+"dvrcam.info",
+"dynns.com",
+"eating-organic.net",
+"fantasyleague.cc",
+"geekgalaxy.com",
+"golffan.us",
+"health-carereform.com",
+"homesecuritymac.com",
+"homesecuritypc.com",
+"hopto.me",
+"ilovecollege.info",
+"loginto.me",
+"mlbfan.org",
+"mmafan.biz",
+"myactivedirectory.com",
+"mydissent.net",
+"myeffect.net",
+"mymediapc.net",
+"mypsx.net",
+"mysecuritycamera.com",
+"mysecuritycamera.net",
+"mysecuritycamera.org",
+"net-freaks.com",
+"nflfan.org",
+"nhlfan.net",
+"no-ip.ca",
+"no-ip.co.uk",
+"no-ip.net",
+"noip.us",
+"onthewifi.com",
+"pgafan.net",
+"point2this.com",
+"pointto.us",
+"privatizehealthinsurance.net",
+"quicksytes.com",
+"read-books.org",
+"securitytactics.com",
+"serveexchange.com",
+"servehumour.com",
+"servep2p.com",
+"servesarcasm.com",
+"stufftoread.com",
+"ufcfan.org",
+"unusualperson.com",
+"workisboring.com",
+"3utilities.com",
+"bounceme.net",
+"ddns.net",
+"ddnsking.com",
+"gotdns.ch",
+"hopto.org",
+"myftp.biz",
+"myftp.org",
+"myvnc.com",
+"no-ip.biz",
+"no-ip.info",
+"no-ip.org",
+"noip.me",
+"redirectme.net",
+"servebeer.com",
+"serveblog.net",
+"servecounterstrike.com",
+"serveftp.com",
+"servegame.com",
+"servehalflife.com",
+"servehttp.com",
+"serveirc.com",
+"serveminecraft.net",
+"servemp3.com",
+"servepics.com",
+"servequake.com",
+"sytes.net",
+"webhop.me",
+"zapto.org",
 "nyc.mn",
 "nid.io",
 "operaunite.com",
 "outsystemscloud.com",
+"ownprovider.com",
+"oy.lc",
+"pgfog.com",
+"pagefrontapp.com",
 "art.pl",
 "gliwice.pl",
 "krakow.pl",
 "poznan.pl",
 "wroc.pl",
 "zakopane.pl",
+"pantheonsite.io",
+"gotpantheon.com",
+"mypep.link",
+"xen.prgmr.com",
 "priv.at",
+"chirurgiens-dentistes-en-france.fr",
+"qa2.com",
+"rackmaze.com",
+"rackmaze.net",
 "rhcloud.com",
+"hzc.io",
+"sandcats.io",
+"logoip.de",
+"logoip.com",
+"biz.ua",
+"co.ua",
+"pp.ua",
+"myshopblocks.com",
 "sinaapp.com",
 "vipsinaapp.com",
 "1kapp.com",
+"bounty-full.com",
+"alpha.bounty-full.com",
+"beta.bounty-full.com",
+"spacekit.io",
+"diskstation.me",
+"dscloud.biz",
+"dscloud.me",
+"dscloud.mobi",
+"dsmynas.com",
+"dsmynas.net",
+"dsmynas.org",
+"familyds.com",
+"familyds.net",
+"familyds.org",
+"i234.me",
+"myds.me",
+"synology.me",
 "gda.pl",
 "gdansk.pl",
 "gdynia.pl",
 "med.pl",
 "sopot.pl",
+"bloxcms.com",
+"townnews-staging.com",
+"tuxfamily.org",
 "hk.com",
 "hk.org",
 "ltd.hk",
 "inc.hk",
+"router.management",
 "yolasite.com",
 "za.net",
 "za.org",
